#include <fits.h>
#include "wfirstpsfdata.h"

ftype fits;
int MODE=0;

void getpsf(char*filt) {
   int x,y,z;
   int x1,y1,x2,y2,ct;
   FILE *f;
   char fn[321];
   float **psf;
   double norm;

   sprintf(fn,"%s/wfirst/data/%s.%s.psf",BASEDIR,filt,wfirst_cn);
   if ((f=fopen(fn,"wb"))==NULL) {
      printf("Cannot write %s\n",fn);
      exit(1);
   }
   psf=(float**)calloc(2*wfirst_rpsf+1,sizeof(float*));
   if (!psf) merr();
   psf+=wfirst_rpsf;
   for (z=-wfirst_rpsf;z<=wfirst_rpsf;z++) {
      psf[z]=(float*)calloc(2*wfirst_rpsf+1,sizeof(float));
      if (!psf[z]) merr();
      psf[z]+=wfirst_rpsf;
   }
   for (ct=0;ct<wfirst_npsfpos;ct++) {
      int cx,cy;
      if (MODE==1) sprintf(fn,"%s/tmp/WFIRST_%s/%s.psf.fits",BASEDIR,filt,filt);
      else sprintf(fn,"%s/tmp/WFIRST_%s/WFI_SCA01_%s_pos_2048_2048_10x_oversampled.fits",BASEDIR,filt,filt);
      readfits(fn,&fits,0);
      if (fits.Next!=0 || fits.img.Z!=1 || fits.img.X<450 || fits.img.Y<450 || fits.img.X%10 || fits.img.Y%10) {printf("Illegal format of PSF file (must be 450x450 or larger and 10x10 oversampled)\n"); exit(-1);}
      cx = fits.img.X/2;
      cy = fits.img.Y/2;
      // Confirm central four pixels
      for (y=0;y<fits.img.Y;y++) for (x=0;x<fits.img.X;x++) {
	 if ((y!=cy-1 && y!=cy) || (x!=cx-1 && x!=cx)) {
	    if (fits.img.data[0][y][x]>=fits.img.data[0][cy-1][cx-1] || fits.img.data[0][y][x]>=fits.img.data[0][cy-1][cx] || fits.img.data[0][y][x]>=fits.img.data[0][cy][cx-1] || fits.img.data[0][y][x]>=fits.img.data[0][cy][cx]) {
	       printf("Error: %d,%d not center (%d,%d higher)\n",cx,cy,x,y);
	       exit(-1);
	    }
	 }
      }
      norm=0;
      for (y=0;y<fits.img.Y;y++) for (x=0;x<fits.img.X;x++) norm+=fits.img.data[0][y][x];
      printf("%3d: total PSF=%f\n",ct,norm);
      // loop over subpixel phasing (x,y)
      for (y=-wfirst_n2psf;y<=wfirst_n2psf;y++) for (x=-wfirst_n2psf;x<=wfirst_n2psf;x++) {
	 for (y1=-wfirst_rpsf;y1<=wfirst_rpsf;y1++) for (x1=-wfirst_rpsf;x1<=wfirst_rpsf;x1++) psf[y1][x1]=0;
	 // simple -- no charge diffusion or delta QE
	 // assume 10x10 subsampled raw image (regardless of wfirst_sub)
	 // assume wfirst_sub=5 so get 2*x and 2*y math below
	 for (y1=0;y1<fits.img.Y;y1++) for (x1=0;x1<fits.img.X;x1++) {
	    x2=(x1-cx+5+2*x+1000*10)/10-1000;
	    y2=(y1-cy+5+2*y+1000*10)/10-1000;
	    if (x2>=-wfirst_rpsf && x2<=wfirst_rpsf && y2>=-wfirst_rpsf && y2<=wfirst_rpsf)
	    psf[y2][x2] += fits.img.data[0][y1][x1];
	 }
	 for (y1=-wfirst_rpsf;y1<=wfirst_rpsf;y1++) ffwrite(psf[y1]-wfirst_rpsf,2*wfirst_rpsf+1,sizeof(float),f);
      }
   }
   for (z=-wfirst_rpsf;z<=wfirst_rpsf;z++) free(psf[z]-wfirst_rpsf);
   free(psf-wfirst_rpsf);
   fclose(f);
   return;
}

int main(int argc,char**argv) {
   int nfilt=0,i;
   char filt[100][20];

   if (argc<2) {
      printf("Usage: %s <filters> <<opts>>\n",*argv);
      printf("   -v2 to use newer input files\n");
      return -1;
   }
   for (i=1;i<argc;i++) {
      if (!strcmp(argv[i],"R062") || !strcmp(argv[i],"Z087") || !strcmp(argv[i],"Y106") || !strcmp(argv[i],"J129") || !strcmp(argv[i],"H158") || !strcmp(argv[i],"F184")) strcpy(filt[nfilt++],argv[i]);
      else if (!strcasecmp(argv[i],"-v2")) MODE=1;
      else {
	 printf("Unknown option: \"%s\"\n",argv[i]);
	 return -1;
      }
   }
   for (i=0;i<nfilt;i++) getpsf(filt[i]);
   return 0;
}
