#include <fits.h>
#include "wfirstpsfdata.h"

ftype fits,pam;
double RN,EXP,EXP0,EPOCH,FIXED_ET=-1.,FIXED_RN=-1.;
float DMIN,DMAX;
int NCOMBINE,Next0;

int WFIRSTtype(ftype *f) {
   if (Next0==0) {printf("**No extension\n"); exit(-1);}
   if (strcmp(getcardval(fits.ext,"EXTNAME",1),"WFIRST-WFI") && strcmp(getcardval(fits.ext,"EXTNAME",1),"SCA01")) {printf("**Format error (extname)\n"); exit(-1);}

   if (Next0==1 && f->ext[0].X==4096 && f->ext[0].Y==4096 && f->ext[0].Z==1) return 1;

   return 0;
}

double getEitherValue(char*str) {
   char *ptr = getcardval(&(fits.img),str,0);
   if (*ptr==0) ptr = getcardval(fits.ext,str,1);
   if (*ptr==0) exit(-1);
   return atof(ptr);
}

void WFIRSTgetcards(void) {
   if (FIXED_ET>0) EXP=FIXED_ET;
   else EXP = getEitherValue("EXPTIME");
   if (FIXED_RN>0) RN=FIXED_RN;
   else RN = getEitherValue("RDNOISE");
   DMIN = -1.e29;
   DMAX = 1.e30;
   EPOCH = 0.0;
   NCOMBINE = 1;
   EXP0=EXP/NCOMBINE;
   return;
}

/*
void WFIRST_DQ_mask(ftype*f,int ext,int drz,int ir) {
   int x,y,dq,i;
   for (y=0;y<f->ext[ext].Y;y++) for (x=0;x<f->ext[ext].X;x++) {
      dq=(int)(f->ext[ext+2].data[0][y][x]+0.5);
      // 1024 unused at present
      if (dq&256) f->ext[ext].data[0][y][x]=safeup(DMAX); // 256
      else if ((dq&512) && USE_FLAT_FLAG) f->ext[ext].data[0][y][x]=safedown(DMIN);
      else if (dq&6335) f->ext[ext].data[0][y][x]=safedown(DMIN); // 4096+2048+128+63
      else if ((dq&8192) && MASKCR) f->ext[ext].data[0][y][x]=safedown(DMIN);
   }
   return;
}
*/

/*
void WFIRST_PAM_mult(int ext,char*pamfn,double mult) {
   int x,y;
   float DMIN1,DMAX1;
   char fn[321];

   if (DMAX>0.) DMAX1=1.5*DMAX;
   else DMAX1=0.;
   if (DMIN<0.) DMIN1=1.5*DMIN;
   else DMIN1=0.;
   sprintf(fn,"%s/wfirst/data/%s",BASEDIR,pamfn);
   readfits(fn,&pam,0);
   if (pam.Next!=1 || pam.ext[0].X<fits.ext[ext].X+offsetx || pam.ext[0].Y<fits.ext[ext].Y+offsety || pam.ext[0].Z!=fits.ext[ext].Z) {
      printf("Error in PAM file %s\n",pamfn);
      exit(-1);
      return;
   }
   for (y=0;y<fits.ext[ext].Y;y++) for (x=0;x<fits.ext[ext].X;x++) if (fits.ext[ext].data[0][y][x]>DMIN && fits.ext[ext].data[0][y][x]<DMAX) {
      fits.ext[ext].data[0][y][x]*=pam.ext[0].data[0][y+offsety][x+offsetx]/mult;
      fits.ext[ext+1].data[0][y][x]*=pam.ext[0].data[0][y+offsety][x+offsetx]/mult;
   }
   else if (fits.ext[ext].data[0][y][x]>=DMAX) fits.ext[ext].data[0][y][x]=safeup(DMAX1);
   else fits.ext[ext].data[0][y][x]=safedown(DMIN1);
   freefits(&pam);
   DMIN=DMIN1;
   DMAX=DMAX1;
   return;
}
*/

/*
void WFIRST_exptime_mult(int ext,double mult) {
   int x,y;

   DMAX*=EXP/mult;
   DMIN*=EXP/mult;
   for (y=0;y<fits.ext[ext].Y;y++) for (x=0;x<fits.ext[ext].X;x++) {
      fits.ext[ext].data[0][y][x]*=EXP/mult;
      fits.ext[ext+1].data[0][y][x]*=EXP/mult;
   }
   return;
}
*/

void WFIRSTsetcards(int ext) {
   insertcards(fits.ext+ext,1.,RN*sqrt(NCOMBINE),EXP,DMIN,DMAX,EPOCH,0.0,EXP0);
   return;
}

int main(int argc,char**argv) {
   int i,tp;

   if (argc<2) {
      printf("Usage: %s <<-flags>> <fits files>\n",*argv);
      printf(" -exptime=#   overrides exposure time\n");
      printf(" -rdnoise=#   overrides readout noise\n");
      return 1;
   }
   for (i=1;i<argc;i++) if (!strncasecmp(argv[i],"-EXPTIME=",9)) FIXED_ET=atof(argv[i]+9);
   else if (!strncasecmp(argv[i],"-RDNOISE=",9)) FIXED_RN=atof(argv[i]+9);
   else {
      readfits(argv[i],&fits,1);
      Next0=fits.Next;
      tp=WFIRSTtype(&fits);
      if (Next0>0 && strcmp(getcardval(fits.ext,"DOL_WFST",0),"")) printf("%s already run through wfirstmask\n",argv[i]);
      else if (tp!=1) printf("%s is not a WFIRST fits file\n",argv[i]);
      else {
	 WFIRSTgetcards();
	 //WFIRST_DQ_mask(&fits,0,0,1);
	 //WFIRST_PAM_mult(0,"ir_wfirst_map.fits",wfirst_ctmult[0]);
	 //WFIRST_exptime_mult(0,1.0);
	 WFIRSTsetcards(0);
	 addcard(fits.ext,"DOL_WFST=                    0 / DOLPHOT WFIRST tag                             ");
	 writefits(argv[i],&fits,1);
      }
      freefits(&fits);
   }
   return 0;
}
